//+------------------------------------------------------------------+
//|                                                         eSCH.mq4 |
//|                                                Sergei Vladimirov |
//|          https://www.mql5.com/ru/users/sergey_mechanic/portfolio |
//+------------------------------------------------------------------+
#property copyright "Sergei Vladimirov"
#property link      "https://www.mql5.com/ru/users/sergey_mechanic/portfolio"
#property version   "3.40"
#property strict

#resource "\\Indicators\\iSCH.ex4"
const string g_sIndName = "::Indicators\\iSCH.ex4";


#define TIME_ERR_VAL 86401
#define TIME_EMPTY_VAL 86400

struct SOrder
{
	int nTicket;
	double fOpenPrice;
	double fSL;
	double fTP;
};

enum ENUM_SIGNAL_BAR
{
	SIGNAL_LIVE_BAR = 0,			//  
	SIGNAL_CLOSED_BAR = 1		//  
};

enum ENUM_AFTER_MAX_ORDERS
{
	CONTINUE_LAST_LOT = 0,		//   
	STOP_ORDERS = 1				//   
};

enum ENUM_SL_MODE
{
	SL_INPUT_VALUE = 0,				//  
	SL_PERCENTS_OF_CHANNEL = 1		//     
};

enum ENUM_TP_MODE
{
	TP_INPUT_VALUE = 0,			//  
	TP_SL_MULTIPLIER = 1			//  * 
};

enum ENUM_SLTP_MODE
{
	SLTP_IMMEDIATE = 0,			//   
	SLTP_ECN = 1					//   
};

enum ENUM_PIPS_MULTIPLIER
{
	PIPSx1 = 1,						// 1
	PIPSx10 = 10					// 10
};

enum ENUM_SIGNAL_DIRECTION
{
	SIGNAL_NORMAL = 0,			//   
	SIGNAL_OPPOSITE = 1			//   
};

// inputs
input ENUM_TIMEFRAMES				i_ePeriod							= PERIOD_CURRENT;												//  
sinput int								i_nMagic								= 347;															//  (Magic Number)
sinput string							i_sCloseTime						= "23:30";														//      (  = .)
input bool								i_bContinueYestSeria				= true;															//     
input double							i_fInitialLot						= 0.1;															//  
input double							i_fLotsMultiplier					= 1;																//  
input uint								i_nMaxOrders						= 10;																//    
input ENUM_AFTER_MAX_ORDERS		i_eAfterMaxOrders					= CONTINUE_LAST_LOT;											//    
input ENUM_SL_MODE					i_eSLMode							= SL_PERCENTS_OF_CHANNEL;									//  stop loss
input uint								i_nSL									= 50;																// Stop Loss, .
input double							i_fSLPercent						= 100;															// Stop Loss, %  
input ENUM_TP_MODE					i_eTPMode							= TP_SL_MULTIPLIER;											//  take profit
input uint								i_nTP									= 50;																// Take Profit, .
input double							i_fTPMultiplier					= 2;																//  take profit
input ENUM_SLTP_MODE					i_eSLTPMode							= SLTP_ECN;														//     
input ENUM_PIPS_MULTIPLIER			i_ePipsMultiplier					= PIPSx10;														//   ( 5-   10)

sinput string							i_sOrder2Separator				= "";																//
sinput string							i_sOrder2IndicarotSettings		= "===   ===";						//
input bool								i_bUseOppositeOrder				= true;															//   
input bool								i_bCloseOnNewMainOrder			= true;															//     
sinput int								i_nMagic2							= 348;															//  (Magic Number)
input double							i_fLotsMultiplier2				= 2;																//  
input uint								i_nMaxOrders2						= 10;																//    
input ENUM_AFTER_MAX_ORDERS		i_eAfterMaxOrders2				= CONTINUE_LAST_LOT;											//    
input ENUM_SL_MODE					i_eSLMode2							= SL_PERCENTS_OF_CHANNEL;									//  stop loss
input ENUM_TP_MODE					i_eTPMode2							= TP_SL_MULTIPLIER;											//  take profit
input uint								i_nSL2								= 50;																// Stop Loss, .
input double							i_fSL2Percent						= 100;															// Stop Loss, %  
input uint								i_nTP2								= 50;																// Take Profit, .
input double							i_fTPMultiplier2					= 2;																//  take profit

sinput string							i_sIndicatorSeparator			= "";																//
sinput string							i_sIndicarotSettings				= "===   ===";						//
sinput string							i_sIndStartTime					= "00:00";														//   
sinput string							i_sIndEndTime						= "09:00";														//   
input ENUM_SIGNAL_BAR				i_eIndSignalBar					= SIGNAL_LIVE_BAR;											//  
input ENUM_SIGNAL_DIRECTION		i_eIndSignalDir					= SIGNAL_NORMAL;												//   

// globals
datetime g_dtCloseTime;
datetime g_dtIndicatorEndTime;
string g_sGlobalVarPrefix;
string g_sLastClosedTicketGV;

datetime g_dtLastSignalTime;
int g_nLastMainTicket;

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
{
	g_dtCloseTime = ParseTimeString(i_sCloseTime);
	g_dtIndicatorEndTime = ParseTimeString(i_sIndEndTime);
	
	//   
	g_sGlobalVarPrefix = WindowExpertName() + "_" + Symbol() + "_" + IntegerToString(i_nMagic) + "_";
	
	//          
	g_sLastClosedTicketGV = g_sGlobalVarPrefix + "LastClosedTicket";
	
	g_dtLastSignalTime = 0;
	g_nLastMainTicket = 0;
	
	for(int i = 0; i < OrdersTotal(); i++)
	{
		if(OrderSelect(i, SELECT_BY_POS) == true)
		{
			if(OrderSymbol() == Symbol() && OrderMagicNumber() == i_nMagic)
			{
				if(OrderType() == OP_BUY || OrderType() == OP_SELL)
				{
					g_nLastMainTicket = OrderTicket();
					break;
				}
			}
		}
	}
	
   return(INIT_SUCCEEDED);
}

//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{

}

//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
{
	bool bNewMainOrder = false;
	
	//     
	SOrder oBuy;
	SOrder oSell;
	SOrder oBuyStop;
	SOrder oSellStop;

	ZeroMemory(oBuy);
	ZeroMemory(oSell);
	ZeroMemory(oBuyStop);
	ZeroMemory(oSellStop);

	int nMarketIndex = 0;
	double fMarketLots = 0;
	int nMarketType = 0;
	
	for(int i = 0; i < OrdersTotal(); i++)
	{
		if(OrderSelect(i, SELECT_BY_POS) == true)
		{
			if(OrderSymbol() == Symbol() && OrderMagicNumber() == i_nMagic)
			{
				switch(OrderType())
				{
				case OP_BUY:
					oBuy.nTicket = OrderTicket();
					oBuy.fOpenPrice = OrderOpenPrice();
					oBuy.fSL = OrderStopLoss();
					oBuy.fTP = OrderTakeProfit();
					nMarketIndex = OrderIndexFromComment(OrderComment());
					fMarketLots = OrderLots();
					nMarketType = OrderType();
					break;
				case OP_SELL:
					oSell.nTicket = OrderTicket();
					oSell.fOpenPrice = OrderOpenPrice();
					oSell.fSL = OrderStopLoss();
					oSell.fTP = OrderTakeProfit();
					nMarketIndex = OrderIndexFromComment(OrderComment());
					fMarketLots = OrderLots();
					nMarketType = OrderType();
					break;
				case OP_BUYSTOP:
					oBuyStop.nTicket = OrderTicket();
					break;
				case OP_SELLSTOP:
					oSellStop.nTicket = OrderTicket();
					break;
				}
			}
		}
	}
	
	//     
	int nMarketTicket = oBuy.nTicket + oSell.nTicket;
	
	if(nMarketTicket > g_nLastMainTicket)
	{
		bNewMainOrder = true;
		g_nLastMainTicket = nMarketTicket;
	}
	
	//      
	SOrder oBuy2;
	SOrder oSell2;
	SOrder oBuyStop2;
	SOrder oSellStop2;
	
	ZeroMemory(oBuy2);
	ZeroMemory(oSell2);
	ZeroMemory(oBuyStop2);
	ZeroMemory(oSellStop2);
	
	int nMarketIndex2 = 0;
	double fMarketLots2 = 0;

	if(i_bUseOppositeOrder)
	{
		for(int i = 0; i < OrdersTotal(); i++)
		{
			if(OrderSelect(i, SELECT_BY_POS) == true)
			{
				if(OrderSymbol() == Symbol() && OrderMagicNumber() == i_nMagic2)
				{
					switch(OrderType())
					{
					case OP_BUY:
						oBuy2.nTicket = OrderTicket();
						oBuy2.fOpenPrice = OrderOpenPrice();
						oBuy2.fSL = OrderStopLoss();
						oBuy2.fTP = OrderTakeProfit();
						nMarketIndex2 = OrderIndexFromComment(OrderComment());
						fMarketLots2 = OrderLots();
						break;
					case OP_SELL:
						oSell2.nTicket = OrderTicket();
						oSell2.fOpenPrice = OrderOpenPrice();
						oSell2.fSL = OrderStopLoss();
						oSell2.fTP = OrderTakeProfit();
						nMarketIndex2 = OrderIndexFromComment(OrderComment());
						fMarketLots2 = OrderLots();
						break;
					case OP_BUYSTOP:
						oBuyStop2.nTicket = OrderTicket();
						break;
					case OP_SELLSTOP:
						oSellStop2.nTicket = OrderTicket();
						break;
					}
				}
			}
		}
	}
	
	//         
	if(i_bCloseOnNewMainOrder)
	{
		if(bNewMainOrder || (oBuy.nTicket == 0 && oSell.nTicket == 0))
		{
			//    
			if(oBuyStop2.nTicket > 0 || oSellStop2.nTicket > 0)
			{
				if(DeleteAllPendings(i_nMagic2) == true)
				{
					ZeroMemory(oBuyStop2);
					ZeroMemory(oSellStop2);
				}
			}
			
			//   
			if(oBuy2.nTicket > 0 || oSell2.nTicket > 0)
			{
				if(CloseAll(i_nMagic2) == true)
				{
					ZeroMemory(oBuy2);
					ZeroMemory(oSell2);
				}
			}
		}
	}
	
	//        (         )
	if(g_dtCloseTime != TIME_EMPTY_VAL && (TimeCurrent() % 86400) >= g_dtCloseTime)
	{
		if(DeleteAllPendings(i_nMagic) == true)
		{
			ZeroMemory(oBuyStop);
			ZeroMemory(oSellStop);
		}
		
		if(CloseAll(i_nMagic, g_sLastClosedTicketGV) == true)
		{
			ZeroMemory(oBuy);
			ZeroMemory(oSell);
			
			nMarketIndex = 0;
		}
		
		if(DeleteAllPendings(i_nMagic2) == true)
		{
			ZeroMemory(oBuyStop2);
			ZeroMemory(oSellStop2);
		}
		
		if(CloseAll(i_nMagic2) == true)
		{
			ZeroMemory(oBuy2);
			ZeroMemory(oSell2);
			
			nMarketIndex2 = 0;
		}
		
		return;
	}
	
	//    ,     -   
	if(oBuy.nTicket == 0 && oSell.nTicket == 0)
	{
		if(DeleteAllPendings(i_nMagic) == true)
		{
			ZeroMemory(oBuyStop);
			ZeroMemory(oSellStop);
		}
		else
			return;
	}
	
	//     
	if(oBuy2.nTicket == 0 && oSell2.nTicket == 0)
	{
		if(DeleteAllPendings(i_nMagic2) == true)
		{
			ZeroMemory(oBuyStop2);
			ZeroMemory(oSellStop2);
		}
		else
			return;
	}
	
	//    ,    
	if(oBuy.nTicket == 0 && oSell.nTicket == 0 && oBuy2.nTicket == 0 && oSell2.nTicket == 0)
	{
		//       ,      ,     
		if((TimeCurrent() % 86400) >= g_dtIndicatorEndTime && g_dtLastSignalTime < (TimeCurrent() - TimeCurrent() % 86400))
		{
			//  ,        
			if(LastMarketOrderBarTime(i_nMagic, i_ePeriod) < iTime(Symbol(), i_ePeriod, 0))
			{
				bool bBuySignal = false;
				bool bSellSignal = false;
				
				//   
				if(i_eIndSignalDir == SIGNAL_NORMAL)
				{
					bBuySignal  = iCustom(Symbol(), i_ePeriod, g_sIndName, i_sIndStartTime, i_sIndEndTime, 0, i_eIndSignalBar) > 0;
					bSellSignal = iCustom(Symbol(), i_ePeriod, g_sIndName, i_sIndStartTime, i_sIndEndTime, 1, i_eIndSignalBar) > 0;
				}
				else
				{
					bBuySignal  = iCustom(Symbol(), i_ePeriod, g_sIndName, i_sIndStartTime, i_sIndEndTime, 1, i_eIndSignalBar) > 0;
					bSellSignal = iCustom(Symbol(), i_ePeriod, g_sIndName, i_sIndStartTime, i_sIndEndTime, 0, i_eIndSignalBar) > 0;
				}
				
				if(bBuySignal || bSellSignal)
					g_dtLastSignalTime = TimeCurrent();
				
				if((bBuySignal && !bSellSignal) || (bSellSignal && !bBuySignal))
				{
					//      ,
					//            
					double fLot = i_fInitialLot;
					int nNextIndex = 1;
					double fSL = 0.0;
					double fTP = 0.0;
					
					if(i_bContinueYestSeria == true)
					{
						if(GlobalVariableCheck(g_sLastClosedTicketGV) == true)
						{
							int nLastClosedTicket = (int) GlobalVariableGet(g_sLastClosedTicketGV);
							
							if(nLastClosedTicket > 0 && OrderSelect(nLastClosedTicket, SELECT_BY_TICKET) == true)
							{
								fLot = OrderLots();
								nNextIndex = OrderIndexFromComment(OrderComment());
							}
						}
					}
					
					fLot = RoundLot(Symbol(), fLot);
					
					RefreshRates();
					
					//  
					if(bBuySignal)
					{
						if(i_eSLTPMode == SLTP_IMMEDIATE)
							CalculateSLTP(OP_BUY, Ask, fSL, fTP);
						
						oBuy.nTicket = MarketOrder(Symbol(), OP_BUY, fLot, 200, fSL, fTP, "=" + IntegerToString(nNextIndex) + "=", i_nMagic, 0, clrGreen);
						
						if(oBuy.nTicket > 0)
						{
							if(OrderSelect(oBuy.nTicket, SELECT_BY_TICKET) == true)
							{
								oBuy.fOpenPrice = OrderOpenPrice();
								oBuy.fSL = OrderStopLoss();
								oBuy.fTP = OrderTakeProfit();
							}
							
							GlobalVariableDel(g_sLastClosedTicketGV);
							nMarketIndex = nNextIndex;
							fMarketLots = fLot;
							nMarketType = OP_BUY;
							
							g_nLastMainTicket = oBuy.nTicket;
							bNewMainOrder = true;
						}
					}
					
					if(bSellSignal)
					{
						if(i_eSLTPMode == SLTP_IMMEDIATE)
							CalculateSLTP(OP_SELL, Bid, fSL, fTP);
						
						oSell.nTicket = MarketOrder(Symbol(), OP_SELL, fLot, 200, fSL, fTP, "=" + IntegerToString(nNextIndex) + "=", i_nMagic, 0, clrGreen);
						
						if(oSell.nTicket > 0)
						{
							if(OrderSelect(oSell.nTicket, SELECT_BY_TICKET) == true)
							{
								oSell.fOpenPrice = OrderOpenPrice();
								oSell.fSL = OrderStopLoss();
								oSell.fTP = OrderTakeProfit();
							}
							
							GlobalVariableDel(g_sLastClosedTicketGV);
							nMarketIndex = nNextIndex;
							fMarketLots = fLot;
							nMarketType = OP_SELL;
							
							g_nLastMainTicket = oSell.nTicket;
							bNewMainOrder = true;
						}
					}
				}
			}
		}
	}
	
	//      ,   
	if(bNewMainOrder && i_bUseOppositeOrder && oBuy2.nTicket == 0 && oSell2.nTicket == 0)
	{
		if(nMarketType == OP_BUY)
		{
			//   
			double fSL2 = 0;
			double fTP2 = 0;
			
			if(i_eSLTPMode == SLTP_IMMEDIATE)
				CalculateSLTP2(OP_SELL, Bid, fSL2, fTP2);
				
			oSell2.nTicket = MarketOrder(Symbol(), OP_SELL, fMarketLots, 200, fSL2, fTP2, "=1=", i_nMagic2, 0, clrOrange);
			
			if(oSell2.nTicket > 0 && OrderSelect(oSell2.nTicket, SELECT_BY_TICKET) == true)
			{
				oSell2.fOpenPrice = OrderOpenPrice();
				oSell2.fSL = OrderStopLoss();
				oSell2.fTP = OrderTakeProfit();
				
				nMarketIndex2 = 1;
				fMarketLots2 = fMarketLots;
			}
		}
		
		if(nMarketType == OP_SELL)
		{
			//   
			double fSL2 = 0;
			double fTP2 = 0;
			
			if(i_eSLTPMode == SLTP_IMMEDIATE)
				CalculateSLTP2(OP_BUY, Ask, fSL2, fTP2);
				
			oBuy2.nTicket = MarketOrder(Symbol(), OP_BUY, fMarketLots, 200, fSL2, fTP2, "=1=", i_nMagic2, 0, clrOrange);
			
			if(oBuy2.nTicket > 0 && OrderSelect(oBuy2.nTicket, SELECT_BY_TICKET) == true)
			{
				oBuy2.fOpenPrice = OrderOpenPrice();
				oBuy2.fSL = OrderStopLoss();
				oBuy2.fTP = OrderTakeProfit();
				
				nMarketIndex2 = 1;
				fMarketLots2 = fMarketLots;
			}
		}
	}
	
	//    
	if(i_eSLTPMode == SLTP_ECN)
	{
		double fSL = 0, fTP = 0;
		
		if(oBuy.nTicket > 0 && oBuy.fSL == 0 && oBuy.fTP == 0)
		{
			if(CalculateSLTP(OP_BUY, oBuy.fOpenPrice, fSL, fTP) == true)
				PlaceSLTP(oBuy.nTicket, fSL, fTP);
		}
		
		if(oSell.nTicket > 0 && oSell.fSL == 0 && oSell.fTP == 0)
		{
			if(CalculateSLTP(OP_SELL, oSell.fOpenPrice, fSL, fTP) == true)
				PlaceSLTP(oSell.nTicket, fSL, fTP);
		}
		
		//     
		if(oBuy2.nTicket > 0 && oBuy2.fSL == 0 && oBuy2.fTP == 0)
		{
			if(CalculateSLTP2(OP_BUY, oBuy2.fOpenPrice, fSL, fTP) == true)
				PlaceSLTP(oBuy2.nTicket, fSL, fTP);
		}
		
		if(oSell2.nTicket > 0 && oSell2.fSL == 0 && oSell2.fTP == 0)
		{
			if(CalculateSLTP2(OP_SELL, oSell2.fOpenPrice, fSL, fTP) == true)
				PlaceSLTP(oSell2.nTicket, fSL, fTP);
		}
	}
	
	//   
	if((oBuy.nTicket > 0 && oSellStop.nTicket == 0) || (oSell.nTicket > 0 && oBuyStop.nTicket == 0))
	{
		if(nMarketIndex < (int)i_nMaxOrders || i_eAfterMaxOrders == CONTINUE_LAST_LOT)
		{
			int nNextIndex = (int)MathMin(nMarketIndex + 1, i_nMaxOrders);
			double fLot = i_fInitialLot * MathPow(i_fLotsMultiplier, nNextIndex - 1);
			fLot = RoundLot(Symbol(), fLot);
			
			if(oBuy.nTicket > 0 && oBuy.fSL > 0 && oBuy.fTP > 0)
			{
				double fPrice = oBuy.fSL;
				double fSL = oBuy.fOpenPrice;
				double fTP = fPrice - (oBuy.fTP - oBuy.fOpenPrice);
				
				PendingOrder(Symbol(), OP_SELLSTOP, fLot, fPrice, 200, fSL, fTP, "=" + IntegerToString(nNextIndex) + "=", 0, i_nMagic, clrNONE);	
			}
			
			if(oSell.nTicket > 0 && oSell.fSL > 0 && oSell.fTP > 0)
			{
				double fPrice = oSell.fSL;
				double fSL = oSell.fOpenPrice;
				double fTP = fPrice + (oSell.fOpenPrice - oSell.fTP);
				
				PendingOrder(Symbol(), OP_BUYSTOP, fLot, fPrice, 200, fSL, fTP, "=" + IntegerToString(nNextIndex) + "=", 0, i_nMagic, clrNONE);	
			}
		}
	}
	
	//     
	if((oBuy2.nTicket > 0 && oSellStop2.nTicket == 0) || (oSell2.nTicket > 0 && oBuyStop2.nTicket == 0))
	{
		if(nMarketIndex2 < (int)i_nMaxOrders2 || i_eAfterMaxOrders2 == CONTINUE_LAST_LOT)
		{
			int nNextIndex2 = (int)MathMin(nMarketIndex2 + 1, i_nMaxOrders2);
			double fLot = fMarketLots * MathPow(i_fLotsMultiplier2, nNextIndex2 - 1);			//  fMarketLots -      
			fLot = RoundLot(Symbol(), fLot);
			
			if(oBuy2.nTicket > 0 && oBuy2.fSL > 0 && oBuy2.fTP > 0)
			{
				double fPrice = oBuy2.fSL;
				double fSL = oBuy2.fOpenPrice;
				double fTP = fPrice - (oBuy2.fTP - oBuy2.fOpenPrice);
				
				PendingOrder(Symbol(), OP_SELLSTOP, fLot, fPrice, 200, fSL, fTP, "=" + IntegerToString(nNextIndex2) + "=", 0, i_nMagic2, clrNONE);	
			}
			
			if(oSell2.nTicket > 0 && oSell2.fSL > 0 && oSell2.fTP > 0)
			{
				double fPrice = oSell2.fSL;
				double fSL = oSell2.fOpenPrice;
				double fTP = fPrice + (oSell2.fOpenPrice - oSell2.fTP);
				
				PendingOrder(Symbol(), OP_BUYSTOP, fLot, fPrice, 200, fSL, fTP, "=" + IntegerToString(nNextIndex2) + "=", 0, i_nMagic2, clrNONE);	
			}
		}
	}
}

//+------------------------------------------------------------------+

// convert intput string HH:MM or HH:MM:SS into seconds from midnight
datetime ParseTimeString(const string sTimeStr)
{
	// error and empty values
	datetime dtErrVal = 86400 + 1;
	datetime dtEmptyVal = 86400;

	// delete spaces from input string
	string sInputString = sTimeStr;
	StringReplace(sInputString, " ", "");
	
	// empty string
	if(sInputString == "")
		return(dtEmptyVal);
	
	// check input string characters: only digits and separator ':' are allowed
	int nLength = StringLen(sInputString);
	char sChar = 0;
	for(int i = 0; i < nLength; i++)
	{
		sChar = (char) StringGetCharacter(sTimeStr, i);
		
		if((sChar < '0' || sChar > '9') && sChar != ':')
			return (dtErrVal);
	}
	
	// array for parsing
	string sSubStrings[];
	ZeroMemory(sSubStrings);
	
	// parse input line
	int nSubStrings = StringSplit(sInputString, ':', sSubStrings);
	
	// format error
	if(nSubStrings < 2 || nSubStrings > 3)
		return(dtErrVal);
		
	int nHour = (int) StringToInteger(sSubStrings[0]);
	int nMinute = (int) StringToInteger(sSubStrings[1]);
	
	int nSeconds = 0;
	
	if(nSubStrings == 3)
		nSeconds = (int) StringToInteger(sSubStrings[2]);
	
	if(nHour < 0 || nHour > 23 || nMinute < 0 || nMinute > 59 || nSeconds < 0 || nSeconds > 59)
		return(dtErrVal);
	
	return( (datetime) (nHour * 3600 + nMinute * 60 + nSeconds) );
}

//+------------------------------------------------------------------+

//        (  "=2=")
int OrderIndexFromComment(const string sOrderComment, const string sDelimeter = "=")
{
	int nIndex = 0;
	
	int nFirstDelimPos = StringFind(sOrderComment, sDelimeter, 0);
	
	if(nFirstDelimPos > -1)
	{
		int nSecondDelimPos = StringFind(sOrderComment, sDelimeter, nFirstDelimPos + 1);
		
		if(nSecondDelimPos > -1)
			nIndex = (int)StringToInteger(StringSubstr(sOrderComment, nFirstDelimPos + 1, nSecondDelimPos - nFirstDelimPos - 1));
	}
	
	return(nIndex);
}

//+------------------------------------------------------------------+
//      (  )
datetime LastMarketOrderBarTime(const int nMagic, const ENUM_TIMEFRAMES eTF)
{
	int nLastMarketTicket = 0;
	datetime dtLastMarketOpenTime = 0;
	
	for(int i = 0; i < OrdersHistoryTotal(); i++)
	{
		if(OrderSelect(i, SELECT_BY_POS, MODE_HISTORY) == true)
		{
			if(OrderSymbol() == Symbol() && OrderMagicNumber() == nMagic)
			{
				if(OrderType() == OP_BUY || OrderType() == OP_SELL)
				{
					if(OrderTicket() > nLastMarketTicket && OrderIndexFromComment(OrderComment()) == 1)
					{
						nLastMarketTicket = OrderTicket();
						dtLastMarketOpenTime = OrderOpenTime();
					}
				}
			}
		}
	}
	
	if(dtLastMarketOpenTime == 0)
		return(0);
	else
		return(dtLastMarketOpenTime - dtLastMarketOpenTime % PeriodSeconds(eTF));
}

//     
bool CalculateSLTP(const int nType, const double fOpenPrice, double & fSL, double & fTP)
{
	fSL = 0;
	fTP = 0;
	
	double fChannelUpperBorder = 0;
	double fChannelLowerBorder = 0;
	
	switch(nType)
	{
	case OP_BUY:
		switch(i_eSLMode)
		{
		case SL_INPUT_VALUE:
			fSL = fOpenPrice - i_nSL * i_ePipsMultiplier * Point();
			break;
		case SL_PERCENTS_OF_CHANNEL:
			fChannelUpperBorder = iCustom(Symbol(), i_ePeriod, g_sIndName, i_sIndStartTime, i_sIndEndTime, 2, 0);
			fChannelLowerBorder = iCustom(Symbol(), i_ePeriod, g_sIndName, i_sIndStartTime, i_sIndEndTime, 3, 0);
			fSL = fChannelUpperBorder - (fChannelUpperBorder - fChannelLowerBorder) * i_fSLPercent / 100;
			break;
		}
		
		switch(i_eTPMode)
		{
		case TP_INPUT_VALUE:
			fTP = fOpenPrice + i_nTP * i_ePipsMultiplier * Point();
			break;
		case TP_SL_MULTIPLIER:
			fTP = fOpenPrice + (fOpenPrice - fSL) * i_fTPMultiplier;
			break;
		}
		
		break;
		
	case OP_SELL:
		switch(i_eSLMode)
		{
		case SL_INPUT_VALUE:
			fSL = fOpenPrice + i_nSL * i_ePipsMultiplier * Point();
			break;
		case SL_PERCENTS_OF_CHANNEL:
			fChannelUpperBorder = iCustom(Symbol(), i_ePeriod, g_sIndName, i_sIndStartTime, i_sIndEndTime, 2, 0);
			fChannelLowerBorder = iCustom(Symbol(), i_ePeriod, g_sIndName, i_sIndStartTime, i_sIndEndTime, 3, 0);
			fSL = fChannelLowerBorder + (fChannelUpperBorder - fChannelLowerBorder) * i_fSLPercent / 100 + (Ask - Bid);
			break;
		}
		
		switch(i_eTPMode)
		{
		case TP_INPUT_VALUE:
			fTP = fOpenPrice - i_nTP * i_ePipsMultiplier * Point();
			break;
		case TP_SL_MULTIPLIER:
			fTP = fOpenPrice - (fSL - fOpenPrice) * i_fTPMultiplier;
			break;
		}
		
		break;
	}
	
	if(fSL <= 0 || fTP <= 0)
	{
		fSL = 0;
		fTP = 0;
		
		return(false);
	}
	
	fSL = NormalizeDouble(fSL, Digits());
	fTP = NormalizeDouble(fTP, Digits());
		
	return(true);
}

//     2
bool CalculateSLTP2(const int nType, const double fOpenPrice, double & fSL, double & fTP)
{
	fSL = 0;
	fTP = 0;
	
	double fChannelUpperBorder = 0;
	double fChannelLowerBorder = 0;
	
	switch(nType)
	{
	case OP_BUY:
		switch(i_eSLMode2)
		{
		case SL_INPUT_VALUE:
			fSL = fOpenPrice - i_nSL2 * i_ePipsMultiplier * Point();
			break;
		case SL_PERCENTS_OF_CHANNEL:
			fChannelUpperBorder = iCustom(Symbol(), i_ePeriod, g_sIndName, i_sIndStartTime, i_sIndEndTime, 2, 0);
			fChannelLowerBorder = iCustom(Symbol(), i_ePeriod, g_sIndName, i_sIndStartTime, i_sIndEndTime, 3, 0);
			fSL = fChannelLowerBorder - (fChannelUpperBorder - fChannelLowerBorder) * i_fSL2Percent / 100;
			break;
		}
		
		switch(i_eTPMode2)
		{
		case TP_INPUT_VALUE:
			fTP = fOpenPrice + i_nTP2 * i_ePipsMultiplier * Point();
			break;
		case TP_SL_MULTIPLIER:
			fTP = fOpenPrice + (fOpenPrice - fSL) * i_fTPMultiplier2;
			break;
		}
		
		break;
		
	case OP_SELL:
		switch(i_eSLMode2)
		{
		case SL_INPUT_VALUE:
			fSL = fOpenPrice + i_nSL2 * i_ePipsMultiplier * Point();
			break;
		case SL_PERCENTS_OF_CHANNEL:
			fChannelUpperBorder = iCustom(Symbol(), i_ePeriod, g_sIndName, i_sIndStartTime, i_sIndEndTime, 2, 0);
			fChannelLowerBorder = iCustom(Symbol(), i_ePeriod, g_sIndName, i_sIndStartTime, i_sIndEndTime, 3, 0);
			fSL = fChannelUpperBorder + (fChannelUpperBorder - fChannelLowerBorder) * i_fSL2Percent / 100 + (Ask - Bid);
			break;
		}
		
		switch(i_eTPMode2)
		{
		case TP_INPUT_VALUE:
			fTP = fOpenPrice - i_nTP2 * i_ePipsMultiplier * Point();
			break;
		case TP_SL_MULTIPLIER:
			fTP = fOpenPrice - (fSL - fOpenPrice) * i_fTPMultiplier2;
			break;
		}
		
		break;
	}
	
	if(fSL <= 0 || fTP <= 0)
	{
		fSL = 0;
		fTP = 0;
		
		return(false);
	}
	
	fSL = NormalizeDouble(fSL, Digits());
	fTP = NormalizeDouble(fTP, Digits());
		
	return(true);
}

//+------------------------------------------------------------------+

bool DeleteAllPendings(const int nMagic)
{
	bool bDone = false;
	
	while(!bDone && !IsStopped())
	{
		bDone = true;
		
		for(int i = OrdersTotal() - 1; i >= 0; i--)
		{
			if(OrderSelect(i, SELECT_BY_POS) == true)
			{
				if(OrderSymbol() == Symbol() && OrderMagicNumber() == nMagic)
				{
					if(OrderType() == OP_BUYSTOP || OrderType() == OP_SELLSTOP)
					{
						if(DeletePendingOrder(OrderTicket(), clrNONE) == false)
							bDone = false;
					}
				}
			}
		}
		
		if(bDone == true)
			return(true);
			
		Sleep(250);
	}
	
	return(false);
}

//+------------------------------------------------------------------+

bool CloseAll(const int nMagic, const string sGVName = NULL)
{
	int nLastClosedTicket = 0;

	bool bDone = false;
	
	while(!bDone && !IsStopped())
	{
		bDone = true;
		
		for(int i = OrdersTotal() - 1; i >= 0; i--)
		{
			if(OrderSelect(i, SELECT_BY_POS) == true)
			{
				if(OrderSymbol() == Symbol() && OrderMagicNumber() == nMagic)
				{
					if(OrderType() == OP_BUY || OrderType() == OP_SELL)
					{
						if(ClosePosition(OrderTicket(), 0, 200, clrRed) == false)
						{
							bDone = false;
						}
						else if(OrderTicket() > nLastClosedTicket)
						{
							nLastClosedTicket = OrderTicket();
						}
					}
				}
			}
		}
		
		if(bDone == true)
		{
			//        
			if(sGVName != NULL && nLastClosedTicket > 0)
				GlobalVariableSet(sGVName, nLastClosedTicket);
			
			return(true);
		}
			
		Sleep(250);
	}
	
	return(false);
}

//+------------------------------------------------------------------+
//									 
//+------------------------------------------------------------------+

//   
double RoundLot(const string sSymbol, const double fLot, const double fArgMinLot = 0, const double fArgMaxLot = 0)
{
	double fMinLot  = SymbolInfoDouble(sSymbol, SYMBOL_VOLUME_MIN);
	double fMaxLot  = SymbolInfoDouble(sSymbol, SYMBOL_VOLUME_MAX);
	double fLotStep = SymbolInfoDouble(sSymbol, SYMBOL_VOLUME_STEP);
	
	// error check
	if(fMinLot <= 0 || fMaxLot <= 0 || fLotStep <= 0)
		return 0;
		
	if(fArgMinLot > 0 && fMinLot < fArgMinLot)
		fMinLot = MathRound(fArgMinLot/fLotStep) * fLotStep;
		
	if(fArgMaxLot > 0 && fLot > fArgMaxLot)
		fMaxLot = MathRound(fArgMaxLot/fLotStep) * fLotStep;
	
	int nLotDigits = 0;
	
	if(fLotStep < 1)
		nLotDigits = (int) MathCeil(-MathLog10(fLotStep));

	double fRoundedLot = MathRound(fLot/fLotStep) * fLotStep;	
	
	if(fRoundedLot < fMinLot)
		fRoundedLot = fMinLot;
		
	if(fRoundedLot > fMaxLot)
		fRoundedLot = fMaxLot;
		
	fRoundedLot = NormalizeDouble(fRoundedLot, nLotDigits);
		
	return(fRoundedLot);
}

//+------------------------------------------------------------------+

int MarketOrder(	const string	sSymbol,
						const int		nType,
						const double	fLot,
						const int		nSlippage		= 200,
						const double	fSL				= 0,
						const double	fTP				= 0,
						const string	sComment			= NULL,
						const int		nMagic			= 0,
						const datetime	dtExpiration	= 0,
						const color		cColor			= clrNONE)
{
	if(nType != OP_BUY && nType != OP_SELL)
	{
		if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
			Print("  -.");
		else
			Print("Wrong type of market order.");
			
		return (-1);
	}
	
	if(SymbolInfoInteger(sSymbol, SYMBOL_VISIBLE) == false)
	{
		if(SymbolSelect(sSymbol, true) == true)
		{
			if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
				Print(" " + sSymbol + "    \" \".");
			else
				Print("Symbol " + sSymbol + " is not found.");
			
			return(-1);
		}
	}
	
	MqlTick oTick;
	int nDigits = (int) SymbolInfoInteger(sSymbol, SYMBOL_DIGITS);
	double fPoint = SymbolInfoDouble(sSymbol, SYMBOL_POINT);
	
	double fPrice = 0;
	double fSLPrice = 0;
	double fTPPrice = 0;
	int nTicket = 0;
	
	// 5 tries to send market order
	int nMaxTries = 5;
	
	for(int k = 0; k < nMaxTries; k++)
	{
		if(SymbolInfoTick(sSymbol, oTick) == false)
		{
			if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
				Print("   ,      \" \".");
			else
				Print("No new quotes or the symbol is not available in the Market Watch window.");
			
			return(-1);
		}
		
		if(AccountFreeMarginCheck(sSymbol, nType, fLot) < 0)
		{
			if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
				Print("    -  " + DoubleToString(fLot, 3) + ".   .");
			else
				Print("Not enough money for market order " + DoubleToString(fLot, 3) + " lots. Expert advisor is stoped.");
				
			return (-1);
		}
		
		if(nType == OP_BUY)
			fPrice = NormalizeDouble(oTick.ask, nDigits);
		else if(nType == OP_SELL)
			fPrice = NormalizeDouble(oTick.bid, nDigits);
		
		if(fSL > 0)
			fSLPrice = NormalizeDouble(fSL, nDigits);
		else
			fSLPrice = 0;
			
		if(fTP > 0)
			fTPPrice = NormalizeDouble(fTP, nDigits);
		else
			fTPPrice = 0;

		ResetLastError();
		
		nTicket = OrderSend(sSymbol, nType, fLot, fPrice, nSlippage, fSLPrice, fTPPrice, sComment, nMagic, dtExpiration, cColor);
		
		if(nTicket > 0)
			break;
			
		if(k >= (nMaxTries - 1))
		{
			
			if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
				Print(" -  .  : " + IntegerToString(GetLastError()));
			else
				Print("Market order failed " + IntegerToString(nMaxTries) + " times. Error code: " + IntegerToString(GetLastError()));
				
			break;
		}
		
		Sleep(250);
	}
	
	return nTicket;
}

//+------------------------------------------------------------------+

bool PlaceSLTP(const int nTicket, const double fSL = 0, const double fTP = 0)
{	
	if(OrderSelect(nTicket, SELECT_BY_TICKET))
	{
		int nType = OrderType();
		
		if(OrderCloseTime() > 0)
		{
			if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
				Print("   SL  TP   #" + IntegerToString(nTicket) + ".   . \r\n");
			else
				Print("PlaceSLTP function: Order #" + IntegerToString(nTicket) + " is closed. \r\n");
				
			return(false);
		}
		
		double fOpenPrice = OrderOpenPrice();
		double fOrderSL = OrderStopLoss();
		double fOrderTP = OrderTakeProfit();
		
		//        
		if( (fSL == 0 || fOrderSL > 0) && (fTP == 0 || fOrderTP) )
			return(true);
			
		//   
		if(nType == OP_BUY || nType == OP_BUYLIMIT || nType == OP_BUYSTOP)
		{
			if((fSL > 0 && fSL >= fOpenPrice) || (fTP > 0 && fTP <= fOpenPrice))
				return(false);
		}
		
		if(nType == OP_SELL || nType == OP_SELLLIMIT || nType == OP_SELLSTOP)
		{
			if((fSL > 0 && fSL <= fOpenPrice) || (fTP > 0 && fTP >= fOpenPrice))
				return(false);
		}
		
		if(fSL < 0 || fTP < 0)
			return(false);
		
		//  
		string sSymbol = OrderSymbol();
		int nDigits = (int) SymbolInfoInteger(sSymbol, SYMBOL_DIGITS);
		
		//      
		double fNewSL, fNewTP;

		if(fOrderSL == 0)
			fNewSL = fSL;
		else
			fNewSL = fOrderSL;
			
		if(fOrderTP == 0)
			fNewTP = fTP;
		else
			fNewTP = fOrderTP;

		//     (3 )
		for(int k = 0; k < 3; k++)
		{
			ResetLastError();
			
			if(OrderModify(nTicket, OrderOpenPrice(), NormalizeDouble(fNewSL, nDigits), NormalizeDouble(fNewTP, nDigits), OrderExpiration(), clrNONE) == true)
				return(true);
				
			if(k == 2)
			{
				if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
					Print("       .  : " + IntegerToString(GetLastError()));
				else
					Print("Take profit or stop loss failed. Error code: " + IntegerToString(GetLastError()));
					
				break;
			}
			
			Sleep(250);
		}
	}
	
	return(false);
}

//+------------------------------------------------------------------+

int PendingOrder(	const string	sSymbol,
						const int		cmd,
						const double	fLot,
						const double	fPrice,
						const int		nSlippage		= 200,
						const double	fSL				= 0,
						const double	fTP				= 0,
						const string	sComment			= NULL,
						const datetime	dtExpiration	= 0,
						const int		nMagic			= 0,
						const color		cColor			= clrNONE)
{
	// check type
	if(cmd != OP_BUYSTOP && cmd != OP_SELLSTOP && cmd != OP_BUYLIMIT && cmd != OP_SELLLIMIT)
		return (-1);
		
	// find symbol
	if(sSymbol != _Symbol && SymbolSelect(sSymbol, true) == false)
	{
		Print("Peinding order failed: symbol " + sSymbol + " is not found. \r\n");
		return (-1);
	}
	
	// symbol spec
	int nDigits = (int) SymbolInfoInteger(sSymbol, SYMBOL_DIGITS);
	double fPoint = SymbolInfoDouble(sSymbol, SYMBOL_POINT);
	
	double fOpenPrice = NormalizeDouble(fPrice, nDigits);
	double fSLPrice = 0;
	double fTPPrice = 0;
	
	// stoplevel flag
	bool bStopLevelAllows = false;
	
	int nTicket = 0;
	
	// 5 tries to send the order
	int nMaxTries = 5;
	
	for(int k = 0; k < nMaxTries && IsTradeAllowed() && !IsStopped(); k++)
	{
		RefreshRates();

		// order prices
		fSLPrice = NormalizeDouble(fSL, nDigits);
		fTPPrice = NormalizeDouble(fTP, nDigits);
		
		// stoplevel check
		double fStopLevel = NormalizeDouble(MarketInfo(sSymbol, MODE_STOPLEVEL) * fPoint, nDigits);
		
		// current prices
		double fBid = NormalizeDouble(SymbolInfoDouble(sSymbol, SYMBOL_BID), nDigits);
		double fAsk = NormalizeDouble(SymbolInfoDouble(sSymbol, SYMBOL_ASK), nDigits);
		
		switch(cmd)
		{
		case OP_BUYSTOP:
			bStopLevelAllows = (fOpenPrice - fAsk) > fStopLevel;
			break;
		case OP_SELLSTOP:
			bStopLevelAllows = (fBid - fOpenPrice) > fStopLevel;
			break;
		case OP_BUYLIMIT:
			bStopLevelAllows = (fAsk - fOpenPrice) > fStopLevel;
			break;
		case OP_SELLLIMIT:
			bStopLevelAllows = (fOpenPrice - fBid) > fStopLevel;
			break;
		}

		ResetLastError();
		
		if(bStopLevelAllows)
		{
			nTicket = OrderSend(sSymbol, cmd, fLot, fOpenPrice, nSlippage, fSLPrice, fTPPrice, sComment, nMagic, dtExpiration, cColor);
			
			// success
			if(nTicket > 0)
				return(nTicket);
		}
			
		// error
		if(k == nMaxTries - 1)
		{
			string sType = "";
			
			switch(cmd)
			{
			case OP_BUYSTOP:
				sType = "BuyStop";
				break;
			case OP_SELLSTOP:
				sType = "SellStop";
				break;
			case OP_BUYLIMIT:
				sType = "BuyLimit";
				break;
			case OP_SELLLIMIT:
				sType = "SellLimit";
				break;
			}
			
			string sError = IntegerToString(GetLastError());
			
			if(!bStopLevelAllows)
				sError = "Stop Level";
			
			if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
				Print("     ", sType, "   ", DoubleToString(fOpenPrice, nDigits), ". : " + sError);
			else
				Print("Pending order ", sType, " at ", DoubleToString(fOpenPrice, nDigits), "failed. Error: " + sError);
			
			return(-1);
		}
		
		Sleep(250);
	}
	
	// if we are here, order failed
	return (-1);
}

//+------------------------------------------------------------------+

bool DeletePendingOrder(const int nTicket, const color cColor = clrNONE)
{
	if(OrderSelect(nTicket, SELECT_BY_TICKET) == true)
	{
		int nType = OrderType();
		
		if((nType != OP_BUYLIMIT && nType != OP_BUYSTOP && nType != OP_SELLLIMIT && nType != OP_SELLSTOP) || OrderCloseTime() > 0)
		{
			if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
				Print("    #" + IntegerToString(nTicket) + ".    ,   . \r\n");
			else
				Print("Order #" + IntegerToString(nTicket) + " was not deleted. The order is not pending or is already deleted. \r\n");
				
			return(false);
		}
		
		int nMaxTries = 3;
		
		for(int k = 0; k < nMaxTries; k++)
		{
			ResetLastError();
			
			if(OrderDelete(nTicket, cColor) == true)
				return(true);
				
			if(k == nMaxTries - 1)
			{
				if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
					Print("     #" + IntegerToString(nTicket) + ".  : " + IntegerToString(GetLastError()));
				else
					Print("Pending order #" + IntegerToString(nTicket) + " is not deleted. Error code: " + IntegerToString(GetLastError()));
					
				return(false);
			}
		}
	}

	return(false);
}

//+------------------------------------------------------------------+

bool ClosePosition(const int nTicket, const double fLot = 0, const int nSlippage = 200, const color cColor = clrNONE)
{
	if(OrderSelect(nTicket, SELECT_BY_TICKET))
	{
		int nDigits = (int) SymbolInfoInteger(OrderSymbol(), SYMBOL_DIGITS);
		
		MqlTick oTick;

		double fCloseLot = OrderLots();
		
		if(fLot > 0)
			fCloseLot = RoundLot(OrderSymbol(), fLot);
		
		// 5 tries to close
		int nMaxTries = 5;
		
		for(int k = 0; k < nMaxTries; k++)
		{
			RefreshRates();
			
			if(SymbolInfoTick(OrderSymbol(), oTick) == false)
			{
				Print("SymbloInfoTick failed");
				return (false);
			}
			
			double fClosePrice = 0;
			
			if(OrderType() == OP_BUY)
				fClosePrice = NormalizeDouble(oTick.bid, nDigits);

			if(OrderType() == OP_SELL)
				fClosePrice = NormalizeDouble(oTick.ask, nDigits);
	
			if(fClosePrice <= 0)
				return(false);
		
			ResetLastError();
			
			if(OrderClose(nTicket, fCloseLot, fClosePrice, nSlippage, cColor) == true)
				return(true);
				
			if(k == nMaxTries - 1)
			{
				if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
					Print("    .  : " + IntegerToString(GetLastError()));
				else
					Print("Market order close failed. Error code: " + IntegerToString(GetLastError()));
					
				return(false);
			}
			
			Sleep(250);
		}
	}
	
	return(false);
}

//+------------------------------------------------------------------+
